﻿/**
 * Copyright (C) 2004 Serguei Pisarev & Maksim Galkin (goldman_alex@mail.ru)
 * You can use this source code in your freeware applications. You cannot change
 * this source code without the express written consent of Serguei Pisarev & Maksim Galkin.
 */
class pseudo.BaseWindow extends pseudo.BaseComponent
{
	private var m_titleBar:pseudo.BaseComponent;
	private var m_pane:pseudo.Pane;
	private var m_btnResize:pseudo.PushButton;
	private var m_nWinX:Number;
	private var m_nWinY:Number;
	private var m_nNormalW:Number;
	private var m_nNormalH:Number;
	private var m_nNormalX:Number;
	private var m_nNormalY:Number;
	private var m_bMaximized:Boolean;
	private var m_nId:Number;
	private var m_bIsResizable:Boolean; // if true - window is resizable
	static private var m_nNewId:Number = 0;

	//
	// PUBLIC METHODS
	//
	static public function create(pOwner:MovieClip, strName:String, nDepth:Number):pseudo.BaseWindow
	{
		pOwner.createEmptyMovieClip(strName, nDepth);
		pOwner[strName].__proto__ = new pseudo.BaseWindow(pOwner[strName]);
		return pOwner[strName];
	}

	static public function getNewId():Number
	{
		return m_nNewId;
	}

	public function BaseWindow(pThis:MovieClip)
	{
		super(pThis);
		if(!pThis) pThis = this;
		m_pClass = pseudo.BaseWindow;

		m_data.className = "BaseWindow";
		m_btnResize = pseudo.PushButton.create(pThis, "m_btnResize", 3);
		m_btnResize.setData({type:"resizeHandle"});
		m_btnResize.addListener("mouseDown", pThis, onStartResize);
		m_btnResize.addListener("mouseUp", pThis, onFinishResize);
		m_btnResize.addListener("rollOver", pThis, onResizeRollOver);
		m_btnResize.addListener("rollOut", pThis, onResizeRollOut);
		m_btnResize.setName("btnResize");

		m_titleBar = m_pane = null;
		m_nWinX = m_nWinY = 0;
		m_nNormalW = m_nNormalH = m_nNormalX = m_nNormalY = 0;
		m_bMaximized = false;
		m_nId = m_nNewId++;
		m_bIsResizable = true;
		m_arrItems.push(m_btnResize);
	}

	public function setPaneSize(nW:Number, nH:Number):Void
	{
		setSize(nW, nH + (m_titleBar ? m_titleBar.getH() : 0));
	}

	public function setLook(look:Object):Void
	{
		m_nH += look.getParams({type:"wndTitle"}).h - m_look.getParams({type:"wndTitle"}).h;
		super.setLook(look);
	}

	public function setIsResizable(bIsResizable:Boolean):Void
	{
		m_bIsResizable = bIsResizable;
		m_btnResize._visible = m_bIsResizable && !m_bMaximized;
	}

	public function getId():Number
	{
		return m_nId;
	}

	public function getMinW():Number
	{
		return Math.max(m_pane.getMinW(), (m_titleBar ? m_titleBar.getMinW() : 0) + 30);
	}

	public function getMinH():Number
	{
		return Math.max(0, m_pane.getMinH() + (m_titleBar ? m_titleBar.getMinH() : 0));
	}

	public function getPrefW():Number
	{
		return Math.max(m_pane.getPrefW(), (m_titleBar ? m_titleBar.getMinW() : 0) + 30);
	}

	public function getPrefH():Number
	{
		return (m_titleBar ? m_titleBar.getPrefH() : 0) + Math.max(0, m_pane.getPrefH());
	}

	public function setData(data:Object, bRecreate:Boolean):Void
	{
		m_titleBar.setData({mode:data.mode});
		super.setData(data, bRecreate);
	}

	public function isFocused():Boolean
	{
		return m_data.mode == "over";
	}

	public function setFocused(bFocused:Boolean):Void
	{
		if(isFocused() != bFocused)
			setData({mode:bFocused ? "over" : "out"});
	}

	public function setTitleBar(pClass:Function):pseudo.BaseMovie
	{
		m_titleBar = pseudo.BaseComponent(addItem(pClass, "titleBar", 1));
		m_titleBar.setData({mode:m_data.mode});
		m_bUpdated = false;
		m_titleBar.addListener("moveWindow", this, moveWindow);
		m_titleBar.addListener("close", this, closeWindow);
		m_titleBar.addListener("maximizeWindow", this, onMaximizeWindow);
		m_titleBar.addListener("minimizeWindow", this, onMinimizeWindow);
		m_titleBar.addListener("hideWindow", this, function(){this.fireEvent("hideWindow");});
		m_titleBar.addListener("startMovePane", this, function(){this.fireEvent("startMovePane");});
		m_titleBar.addListener("finishMovePane", this, function(){this.fireEvent("finishMovePane");});
		m_titleBar.addListener("movePane", this, function(){this.fireEvent("movePane");});
		return m_titleBar;
	}

	public function getTitleBar():pseudo.BaseMovie
	{
		return m_titleBar;
	}

	public function setPane(pClass:Function):pseudo.Pane
	{
		m_bUpdated = false;
		m_pane = pseudo.Pane(addItem(pClass, "pane", 2));
		m_pane.addListener("close", this, closeWindow);
		return m_pane;
	}

	public function getPane():pseudo.Pane
	{
		return m_pane;
	}

	public function isMaximized():Boolean
	{
		return m_bMaximized;
	}

	public function update():Void
	{
		if(!m_bUpdated)
		{
			var obj:Object = m_look.getResizeBtnParams();
			m_btnResize.setSize(obj.w, obj.h);
			m_titleBar.setW(m_nW);
			m_pane.setSize(m_nW, m_nH - (m_titleBar ? m_titleBar.getH() : 0));
			m_pane._y = m_titleBar ? m_titleBar.getH() : 0;
			m_btnResize._x = m_nW - m_btnResize.getW();
			m_btnResize._y = m_nH - m_btnResize.getH();
		}
		super.update();
	}

	public function invokeEvent(strEvent:String):Void
	{
		super.invokeEvent(strEvent);
		switch(strEvent)
		{
			case "maximizeWindow":
				maximizeWindow();
				break;
			case "minimizeWindow":
				minimizeWindow();
				break;
			case "close":
				closeWindow();
				break;
		}
	}

	//
	// PRIVATE METHODS
	//

	private function movieMouseDown(bSkip:Boolean):Void
	{
		if(!bSkip)
		{
			m_nWinX = _x;
			m_nWinY = _y;
		}
		super.movieMouseDown(true);
	}

	private function moveWindow():Void
	{
		if(m_bMouseDown)
			fireEvent(
				"moveWindow",
				m_nWinX + _root._xmouse - m_nLastX,
				m_nWinY + _root._ymouse - m_nLastY
			);
	}

	private function maximizeWindow():Void
	{
		if(!m_bMaximized)
		{
			m_titleBar.invokeEvent("maximizeWindow");
			m_nNormalW = m_nW;
			m_nNormalH = m_nH;
			m_nNormalX = _x;
			m_nNormalY = _y;
			m_bMaximized = true;
			m_btnResize._visible = false;
		}
	}

	private function minimizeWindow():Void
	{
		if(m_bMaximized)
		{
			m_titleBar.invokeEvent("minimizeWindow");
			setSize(m_nNormalW, m_nNormalH);
			_x = m_nNormalX;
			_y = m_nNormalY;
			m_bMaximized = false;
			m_btnResize._visible = m_bIsResizable;
		}
	}

	private function closeWindow():Void
	{
		fireEvent("beforeClose");
		fireEvent("close");
	}

	private function 	onMaximizeWindow():Void
	{
		fireEvent("maximizeWindow");
	}

	private function 	onMinimizeWindow():Void
	{
		fireEvent("minimizeWindow");
	}

	private function onStartResize():Void
	{
		m_btnResize.addListener("mouseMove", this, onResize);
		fireEvent("startResize");
	}

	private function onFinishResize():Void
	{
		m_btnResize.removeListener("mouseMove");
		fireEvent("finishResize");
	}

	private function onResize():Void
	{
		if(m_btnResize.isMouseDown()) fireEvent("resize");
	}

	private function onResizeRollOver():Void
	{
		pseudo.CursorManager.getInstance().setCursor("diagResize1");
	}

	private function onResizeRollOut():Void
	{
		pseudo.CursorManager.getInstance().setCursor();
	}
}
