﻿/**
 * Copyright (C) 2004 Serguei Pisarev & Maksim Galkin (goldman_alex@mail.ru)
 * You can use this source code in your freeware applications. You cannot change
 * this source code without the express written consent of Serguei Pisarev & Maksim Galkin.
 */
/**
 * This class describes cell element. Each cell can have a large number of elements,
 * each element can be placed into the cell in many ways.
 */
class pseudo.Cell extends pseudo.BaseMovie
{
	/** number of rows in cell */
	private var m_nColSpan:Number;
	/** number of cols in cell */
	private var m_nRowSpan:Number;
	/** row index in the table */
	private var m_nRow:Number;
	/** col index in the table */
	private var m_nCol:Number;
	/** array of cell's items */
	private var m_arrItems:Array;
	/** cell color */
	private var m_nColor:Number;
	/** cell alpha */
	private var m_nColorAlpha:Number;

	// PUBLIC METHODS

	/**
	 * Creates cell.
	 * @param pOwner Owner of the cell.
	 * @param strName Cell name.
	 * @param nDepth Cell movie depth.
	 * @return Created cell.
	 */
	static public function create(pOwner:MovieClip, strName:String, nDepth:Number):pseudo.Cell
	{
		pOwner.createEmptyMovieClip(strName, nDepth);
		pOwner[strName].__proto__ = new pseudo.Cell(pOwner[strName]);
		return pOwner[strName];
	}

	/**
	 * Constructor.
	 * @param pThis Pointer to this MovieClip.
	 */
	public function Cell(pThis:MovieClip)
	{
		super(pThis);
		if(!pThis) pThis = this;
		m_pClass = pseudo.Cell;

		m_nW = 10;
		m_nH = 10;
		m_nColSpan = 1;
		m_nRowSpan = 1;
		m_arrItems = new Array();
		m_nColor = null;
		m_nColorAlpha = 100;
	}

	/**
	 * Gets number cols the cell overlaps.
	 * @return Number of cols.
	 */
	public function getColSpan():Number
	{
		return m_nColSpan;
	}

	/**
	 * Gets number of rows the cell overlaps.
	 * @return Number of rows.
	 */
	public function getRowSpan():Number
	{
		return m_nRowSpan;
	}

	public function setColSpan(nColSpan:Number):Void
	{
		m_nColSpan = nColSpan;
	}

	public function setRowSpan(nRowSpan:Number):Void
	{
		m_nRowSpan = nRowSpan;
	}

	/**
	 * Gets column index in the table.
	 * @return Column index.
	 */
	public function getCol():Number
	{
		return m_nCol;
	}

	public function setCol(nCol:Number):Void
	{
		m_nCol = nCol;
	}

	/**
	 * Gets row index in the table.
	 * @return Row index.
	 */
	public function getRow():Number
	{
		return m_nRow;
	}

	public function setRow(nRow:Number):Void
	{
		m_nRow = nRow;
	}

	public function setColor(nColor:Number, nAlpha:Number):Void
	{
		m_nColor = nColor;
		if(nAlpha != undefined)
			m_nColorAlpha = nAlpha;
	}

	public function addItem(pClass:Function):pseudo.BaseMovie
	{
		var nDepth:Number = getNextHighestDepth();
		var item:pseudo.BaseMovie = pClass.create(this, "item" + nDepth, nDepth);
		m_arrItems.push(item);
		item.setLook(m_look);
		return item;
	}

	public function setLook(look:Object):Void
	{
		super.setLook(look);
		for(var i:Number = 0; i < m_arrItems.length; i++)
			m_arrItems[i].setLook(look);
	}

	/**
	 * Updates all items position and size.
	 */
	public function update():Void
	{
		if(!m_bUpdated)
		{
			for(var i:Number = 0; i < m_arrItems.length; i++)
			{
				m_arrItems[i].setSize(m_nW, m_nH);
				m_arrItems[i].update();
			}
			super.update();
		}
	}

	/**
	 * Draws all items.
	 */
	public function draw():Void
	{
		if(!m_bDrawn)
		{
			super.draw();
			if(m_nColor || m_nColor === 0)
				drawRect(0, 0, m_nW, m_nH, m_nColor, m_nColorAlpha);
			for(var i:Number = 0; i < m_arrItems.length; i++)
				m_arrItems[i].draw();
		}
	}

	/**
	 * Removes cell.
	 */
	public function remove():Void
	{
		reset();
		delete m_arrItems;
		super.remove();
		delete this;
	}

	//
	// PRIVATE METHODS
	//

	/**
	 * Resets table. Frees all data.
	 */
	private function reset():Void
	{
		m_nW = 10;
		m_nH = 10;
		m_nColSpan = 0;
		m_nRowSpan = 0;
		for(var i:Number = 0; i < m_arrItems.length; i++)
			m_arrItems[i].remove();
		m_arrItems.splice(0);
	}
}