﻿/**
 * Copyright (C) 2004 Serguei Pisarev & Maksim Galkin (goldman_alex@mail.ru)
 * You can use this source code in your freeware applications. You cannot change
 * this source code without the express written consent of Serguei Pisarev & Maksim Galkin.
 */
/** Describes table class. */
class pseudo.Table extends pseudo.BaseComponent
{
	/** Contains info about columns such as columns' width. */
	private var m_arrCols:Array;
	/** Contains info about rows such as rows' heights. */
	private var m_arrRows:Array;
	/** Array of cells. */
	private var m_arrCells:Array;
	/** Summary width of fixed columns. */
	private var m_nFixedW:Number;
	/** Summary height of fixed rows. */
	private var m_nFixedH:Number;
	/** Number of fixed columns in table. */
	private var m_nFixedCols:Number;
	/** Number of fixed rows in table. */
	private var m_nFixedRows:Number;
	/** Minimum width of 'free' cols. */
	private var m_nMinFreeW:Number;
	/** Minimum height of 'free' rows. */
	private var m_nMinFreeH:Number;
	/** Minimum width of the table. */
	private var m_nMinW:Number;
	/** Minimum height of the table. */
	private var m_nMinH:Number;
	/** Is the table initialized. */
	private var m_bInit:Boolean;

	//
	// PUBLIC METHODS
	//

	/**
	 * Creates table.
	 * @param pOwner Parent of the table.
	 * @param strName Table name.
	 * @param nDepth Table movie depth.
	 * @return Created table.
	 */
	static public function create(pOwner:MovieClip, strName:String, nDepth:Number):pseudo.Table
	{
		pOwner.createEmptyMovieClip(strName, nDepth);
		pOwner[strName].__proto__ = new pseudo.Table(pOwner[strName]);
		return pOwner[strName];
	}

	/**
	 * Constructor. You should create table by calling
	 * pseudo.Table.create(...) instead of calling
	 * this constructor otherwise you can't have movie instance.
	 * @param pThis Pointer to this MovieClip.
	 */
	public function Table(pThis)
	{
		super(pThis);
		if(!pThis) pThis = this;
		m_pClass = pseudo.Table;

		// by default table fits all allowed space
		m_nW = -1;
		m_nH = -1;
		m_nFixedW = 0;
		m_nFixedH = 0;
		m_nFixedCols = 0;
		m_nFixedRows = 0;
		m_nMinFreeW = 0;
		m_nMinFreeH = 0;
		m_arrRows = new Array();
		m_arrCols = new Array();
		m_arrCells = new Array();
		m_bInit = false;
	}

	/**
	 * Sets table look.
	 * @param look New look.
	 */
	public function setLook(look:Object):Void
	{
		super.setLook(look);
		for(var i:Number = 0; i < m_arrCells.length; i++)
			m_arrCells[i].setLook(look);
	}

	/**
	 * Adds column to table.
	 * @param nMinSize Minimal size of column.
	 * @param nSize Ideal size of column. If nSize < 0 then it's proceeded as percent.
	 */
	public function addCol(nMinSize:Number, nSize:Number):Void
	{
		nMinSize = nSize > 0 ? Math.max(nMinSize, 10) : 0;
		m_arrCols.push(new Object({setSize:nSize, defSize:nSize, size:nSize, minSize:nMinSize})) - 1;
		m_bInit = false;
	}

	/**
	 * Adds row to table.
	 * @param nMinSize Minimal size of row.
	 * @param nSize Ideal size of row. If nSize < 0 then it's proceeded as percent.
	 */
	public function addRow(nMinSize:Number, nSize:Number):Void
	{
		nMinSize = nSize > 0 ? Math.max(nMinSize, 10) : 0;
		m_arrRows.push(new Object({setSize:nSize, defSize:nSize, size:nSize, minSize:nMinSize})) - 1;
		m_bInit = false;
	}

	/**
	 * Adds cell to table.
	 * @param nCol Number of column cell belongs to.
	 * @param nRow Number of row cell belongs to.
	 * @param nColSpan Number of columns that are spanned by cell.
	 * @param nRowSpan Number of rows that are spanned by cell.
	 * @return Created cell.
	 */
	public function addCell(nCol:Number, nRow:Number, nColSpan:Number, nRowSpan:Number):pseudo.Cell
	{
		var nDepth:Number = getNextHighestDepth();
		var cell:pseudo.Cell = pseudo.Cell.create(this, "cell" + nDepth, nDepth);
		m_arrCells.push(cell);
		cell.setLook(m_look);
		cell.setColSpan(nColSpan);
		cell.setRowSpan(nRowSpan);
		cell.setCol(nCol);
		cell.setRow(nRow);
		return cell;
	}

	/**
	 * Updates positions and sizes of all items.
	 */
	public function update():Void
	{
		if(!m_bInit) init();
		if(!m_bUpdated)
		{
			var nFreeW:Number = Math.max(m_nW - m_nFixedW, m_nMinFreeW);
			var nFreeH:Number = Math.max(m_nH - m_nFixedH, m_nMinFreeH);
			// reduce freeW
			var nOldFreeW:Number = nFreeW;
			var nMaxCol:Number = -1;
			for(var i:Number = 0; i < m_arrCols.length; i++)
				if(m_arrCols[i].defSize < 0)
				{
					if(nMaxCol < 0) nMaxCol = i; // find col with maximum width
					else if(m_arrCols[i].defSize < m_arrCols[nMaxCol].defSize)
						nMaxCol = i;
					var nScaledW:Number = -m_arrCols[i].defSize * nOldFreeW;
					nFreeW -= Math.max(0, m_arrCols[i].minSize - nScaledW);
				}
			// reduce freeH
			var nOldFreeH:Number = nFreeH;
			var nMaxRow:Number = -1;
			for(var i:Number = 0; i < m_arrRows.length; i++)
				if(m_arrRows[i].defSize < 0)
				{
					if(nMaxRow < 0) nMaxRow = i; // find row with maximum height
					else if(m_arrRows[i].defSize < m_arrRows[nMaxRow].defSize)
						nMaxRow = i;
					var nScaledH:Number = -m_arrRows[i].defSize * nOldFreeH;
					nFreeH -= Math.max(0, m_arrRows[i].minSize - nScaledH);
				}
			// update cols data
			var nNewFreeW:Number = 0;
			var nPrevX:Number = 0;
			for(var i:Number = 0; i < m_arrCols.length; i++)
				if(m_arrCols[i].defSize < 0)
				{
					m_arrCols[i].size =
						Math.max(m_arrCols[i].minSize, -Math.round(m_arrCols[i].defSize * nFreeW));
					nNewFreeW += m_arrCols[i].size; // calculate new width. It should be the same as nOldFreeW
				}
			if(nMaxCol >= 0) m_arrCols[nMaxCol].size += nOldFreeW - nNewFreeW;
			for(var i:Number = 0; i < m_arrCols.length; i++)
			{ // calculate cell x offset
				m_arrCols[i].x = nPrevX;
				nPrevX += m_arrCols[i].size;
			}
			// update rows data
			var nNewFreeH:Number = 0;
			var nPrevY:Number = 0;
			for(var i:Number = 0; i < m_arrRows.length; i++)
				if(m_arrRows[i].defSize < 0)
				{
					m_arrRows[i].size =
						Math.max(m_arrRows[i].minSize, -Math.round(m_arrRows[i].defSize * nFreeH));
					nNewFreeH += m_arrRows[i].size; // calculate new height. It should be the same as nOldFreeH
				}
			if(nMaxRow >= 0) m_arrRows[nMaxRow].size += nOldFreeH - nNewFreeH;
			for(var i:Number = 0; i < m_arrRows.length; i++)
			{ // calculate cell y offset
				m_arrRows[i].y = nPrevY;
				nPrevY += m_arrRows[i].size;
			}
			// update cells
			for(var i:Number = 0; i < m_arrCells.length; i++)
			{
				var nCol:Number = m_arrCells[i].getCol();
				var nRow:Number = m_arrCells[i].getRow();
				m_arrCells[i].setSize( // set new cell size
					getColsSize(nCol, m_arrCells[i].getColSpan()),
					getRowsSize(nRow, m_arrCells[i].getRowSpan())
				);
				m_arrCells[i]._x = m_arrCols[nCol].x; // XXX add offset for cycling cols and/or rows
				m_arrCells[i]._y = m_arrRows[nRow].y;
				m_arrCells[i].update();
			}
		}
		m_nW = getColsSize(0, m_arrCols.length);
		m_nH = getRowsSize(0, m_arrRows.length);
		super.update();
	}

	/**
	 * Draws all items.
	 */
	public function draw():Void
	{
		if(!m_bDrawn)
		{
			for(var i:Number = 0; i < m_arrCells.length; i++)
				m_arrCells[i].draw();
		}
		super.draw();
	}

	public function remove():Void
	{
		reset();
		delete m_arrRows;
		delete m_arrCols;
		delete m_arrCells;
		super.remove();
	}

	//
	// PRIVATE METHODS
	//

	/**
	 * Resets table. Frees all data.
	 */
	private function reset():Void
	{
		for(var i:Number = 0; i < m_arrRows.length; i++)
			delete m_arrRows[i];
		for(var i:Number = 0; i < m_arrCols.length; i++)
			delete m_arrCols[i];
		m_arrRows.splice(0);
		m_arrCols.splice(0);
		for(var i:Number = 0; i < m_arrCells.length; i++)
			m_arrCells[i].remove();
		m_arrCells[i].splice(0);
		m_nFixedW = 0;
		m_nFixedH = 0;
		m_nFixedCols = 0;
		m_nFixedRows = 0;
		m_nMinFreeW = 0;
		m_nMinFreeH = 0;
	}

	/**
	 * Gets summary width of columns from nStart to min(nStart + nSpan, cols number).
	 * @param nStart First column.
	 * @param nSpan Number of columns.
	 */
	private function getColsSize(nStart:Number, nSpan:Number):Number
	{
		var nEnd:Number = Math.min(nStart + nSpan, m_arrCols.length);
		var nSize:Number = 0;
		for(var i = nStart; i < nEnd; i++)
			nSize += m_arrCols[i].size;
		return nSize;
	}

	/**
	 * Gets summary height of rows from nStart to min(nStart + nSpan, cols number).
	 * @param nStart First row.
	 * @param nSpan Number of rows.
	 */
	private function getRowsSize(nStart:Number, nSpan:Number):Number
	{
		var nEnd:Number = Math.min(nStart + nSpan, m_arrRows.length);
		var nSize:Number = 0;
		for(var i = nStart; i < nEnd; i++)
			nSize += m_arrRows[i].size;
		return nSize;
	}

	/**
	 * Inits table.
	 */
	private function init():Void
	{
		m_bInit = true;
		m_nFixedW = 0;
		m_nFixedH = 0;
		m_nFixedCols = 0;
		m_nFixedRows = 0;
		m_nMinFreeW = 0;
		m_nMinFreeH = 0;

		// cols processing
		var nFreeW:Number = 0;
		for(var i:Number = 0; i < m_arrCols.length; i++)
		{ // process all cols
			m_nMinFreeW += m_arrCols[i].minSize;
			if(m_arrCols[i].setSize > 0)
			{ // count number of fixed columns and their summary width
				m_nFixedW += m_arrCols[i].setSize;
				m_nFixedCols++;
			}
			else nFreeW -= m_arrCols[i].setSize;
		}
		m_nMinW = Math.max(m_nMinW, m_nFixedW + m_nMinFreeW);
		// rows processing
		var nFreeH:Number = 0;
		for(var i:Number = 0; i < m_arrRows.length; i++)
		{ // process all rows
			m_nMinFreeH += m_arrRows[i].minSize;
			if(m_arrRows[i].setSize > 0)
			{ // count number of fixed rows and their summary height
				m_nFixedH += m_arrRows[i].setSize;
				m_nFixedRows++;
			}
			else nFreeH -= m_arrRows[i].setSize;
		}
		m_nMinH = Math.max(m_nMinH, m_nFixedH + m_nMinFreeH);

		for(var i:Number = 0; i < m_arrCols.length; i++) // normalize columns percents
			if(m_arrCols[i].setSize < 0) m_arrCols[i].defSize = m_arrCols[i].setSize / nFreeW;
		for(var i:Number = 0; i < m_arrRows.length; i++) // normalize rows percents
			if(m_arrRows[i].setSize < 0) m_arrRows[i].defSize = m_arrRows[i].setSize / nFreeH;
	}
}