﻿/**
 * Copyright (C) 2004 Serguei Pisarev & Maksim Galkin (goldman_alex@mail.ru)
 * You can use this source code in your freeware applications. You cannot change
 * this source code without the express written consent of Serguei Pisarev & Maksim Galkin.
 */
class pseudo.TextArea extends pseudo.BaseComponent
{
	private var m_scrollVert:pseudo.ScrollBar;
	private var m_edit:pseudo.Edit;
	private var m_nBeg:Number;
	private var m_nEnd:Number;

	//
	// PUBLIC METHODS
	//
	static public function create(pOwner:MovieClip, strName:String, nDepth:Number):pseudo.TextArea
	{
		pOwner.createEmptyMovieClip(strName, nDepth);
		pOwner[strName].__proto__ = new pseudo.TextArea(pOwner[strName]);
		return pOwner[strName];
	}

	public function TextArea(pThis:MovieClip)
	{
		super(pThis);
		if(!pThis) pThis = this;
		m_pClass = pseudo.TextArea;

		m_edit = pseudo.Edit.create(pThis, "m_edit", 1);
		m_edit.addListener("scroll", pThis, onTextScrolled);
		m_edit.addListener("setFocus", pThis, changeMode);
		m_edit.addListener("killFocus", pThis, changeMode);
		m_edit.addListener("changed", pThis, onTextScrolled);
		m_edit.setWordWrap(true);
		m_edit.setMultiline(true);
		m_edit.setName("edit");
		m_edit.setData({type:"textArea"});
		m_data.className = "TextArea";
		m_data.type = "edit";
		m_data.mode = "out";
		m_nBeg = m_nEnd = 0;
		m_arrItems.push(m_edit);
	}

	public function getMinW():Number
	{
		return m_edit.getMinW() + (m_scrollVert ? m_scrollVert.getW() : 0);
	}

	public function getMinH():Number
	{
		return m_edit.getMinH();
	}

	public function getText():String
	{
		return m_edit.getText();
	}

	public function setText(strText:String):Void
	{
		m_edit.setText(strText);
		m_bUpdated = false;
	}

	public function getTextW():Number
	{
		return m_edit.getTextW();
	}

	public function getTextH():Number
	{
		return m_edit.getTextH();
	}

	public function setData(data:Object):Void
	{
		super.setData(data);
		m_edit.setData({type:data.type});
	}

	public function getUserW():Number
	{
		return super.getUserW() - (m_scrollVert ? m_scrollVert.getW() : 0);
	}

	public function update(strPart:String):Void
	{
		if(!m_bUpdated)
		{
			var fld:TextField = m_edit.getField();
			m_edit.setSize(m_nW - 3, super.getUserH());
			fld._width = m_edit.getUserW();
			fld._height = m_edit.getUserH();
			fld.text = fld.text;
			if(fld.maxscroll > 1)
			{
				createScrollVert();
				m_scrollVert.setH(m_nH - m_look.getBorder("top", m_data) -
					m_look.getBorder("bottom", m_data));
				m_scrollVert._y = m_look.getBorder("top", m_data);
				m_scrollVert._x = m_nW - m_look.getBorder("right", m_data) - m_scrollVert.getW();
				m_edit.setW(m_scrollVert._x - 3);
				fld._width = m_edit.getUserW();
				m_scrollVert.setRange(1, fld.bottomScroll - fld.scroll + fld.maxscroll + 1);
				m_scrollVert.setScrollSize(fld.bottomScroll - fld.scroll + 1);
			}
			else
			{
				m_scrollVert.remove();
				delete m_scrollVert;
			}
			m_edit._x = m_look.getMargin("left", m_data);
			m_edit._y = m_look.getMargin("top", m_data);
		}
		super.update();
	}

	//
	// PRIVATE METHODS
	//

	private function movieRollOut(bSkip:Boolean):Void
	{
		if(!bSkip) changeMode();
		super.movieRollOut(true);
	}

	private function movieRollOver(bSkip:Boolean):Void
	{
		if(!bSkip) changeMode();
		super.movieRollOver(true);
	}

	private function changeMode():Void
	{
		m_data.mode = m_edit.isFocused() ? "press" : (isMouseOver() ? "over" : "out");
		m_bDrawn = false;
		draw();
	}

	private function onTextScrolled():Void
	{
		m_bUpdated = false;
		var bNoScroll:Boolean = m_scrollVert == undefined;
		update();
		if(bNoScroll != (m_scrollVert == undefined)) draw();
		else m_scrollVert.draw();
		m_scrollVert.setPos(m_edit.getField().scroll);
	}

	private function onChangePos():Void
	{
		m_edit.getField().scroll = m_scrollVert.getPos();
	}

	private function scrollMouseDown():Void
	{
		var nBeg:Number = Selection.getBeginIndex();
		var nEnd:Number = Selection.getEndIndex();
		if(nBeg > -1) m_nBeg = nBeg;
		if(nEnd > -1) m_nEnd = nEnd;
	}

	private function scrollMouseUp():Void
	{
		var nScroll = m_edit.getField().scroll;
		Selection.setFocus(m_edit.getField()._target);
		Selection.setSelection(m_nBeg, m_nEnd);
		m_edit.getField().scroll = nScroll;
	}

	private function createScrollVert():Void
	{
		if(!m_scrollVert)
		{
			m_scrollVert = pseudo.ScrollBar.create(this, "m_scrollVert", 3);
			m_scrollVert.addListener("changePos", this, onChangePos);
			m_scrollVert.addListener("mouseUp", this, scrollMouseUp);
			m_scrollVert.addListener("mouseDown", this, scrollMouseDown);
			m_scrollVert.setW(15);
			m_scrollVert.setSmallScroll(1);
			m_scrollVert.setName("scrollVert");
			m_scrollVert.setLook(m_look);
			m_arrItems.push(m_scrollVert);
		}
	}
}