﻿class pseudo.Tree extends pseudo.List
{
	private var m_arrTreeData:Array;
	private var m_nSelLeaf:Number;

	//
	// PUBLIC METHODS
	//

	/**
	 * Creates tree.
	 * @param pOwner Parent of the tree.
	 * @param strName Tree name.
	 * @param nDepth Tree movie depth.
	 * @return Created tree.
	 */
	static public function create(pOwner:MovieClip, strName:String, nDepth:Number):pseudo.Tree
	{
		pOwner.createEmptyMovieClip(strName, nDepth);
		pOwner[strName].__proto__ = new pseudo.Tree(pOwner[strName]);
		return pOwner[strName];
	}

	/**
	 * Constructor. You should create tree by calling
	 * pseudo.Tree.create(...) instead of calling
	 * this constructor otherwise you don't get movie instance.
	 * @param pThis Pointer to this MovieClip.
	 */
	public function Tree(pThis:MovieClip)
	{
		super(pThis);
		if(!pThis) pThis = this;
		m_pClass = pseudo.Tree;

		m_arrData = new Array();
		addListener("changed", pThis, onChanged);
	}

	/**
	 * Updates tree data. Generates temporary tree structure. By using this
	 * tree adds to m_arrTreeData information about vert. lines visibility.
	 * Also adds to m_arrTreeData previous and next depths values.
	 * This functions should always be called whenever the data values are changed.
	 */
	public function updateTreeData():Void
	{
		var arrLines:Array = new Array();
		var node:Object = getTreeStruct(m_arrTreeData).nodes[0];
		while(node)
		{
			var nodes:Array = node.parent.nodes;
			var item:Object = node.item;
			// is node first
			item.first = node == nodes[0];
			// is node last
			item.last = node == nodes[nodes.length - 1];
			// does the node have children
			if(item.cellType == 1)
				item.hasChildren = node.nodes.length > 0;
			var depth:Number = item.depth;
			if(arrLines.length > depth)
				arrLines.splice(depth);
			arrLines.push(!item.last);
			item.lines = arrLines.slice();
			node = node.next;
		}
		m_bUpdated = false;
	}

	public function setTreeData(arrData:Array)
	{
		m_arrTreeData = arrData;
		m_nSelLeaf = -1;
		updateTreeData();
	}

	public function getTreeData():Array
	{
		return m_arrTreeData;
	}

	public function collapseAll():Void
	{
		m_nOldSel = m_nSelCell = -1;
		for(var i:Number = 0; i < m_arrTreeData.length; i++)
			if(m_arrTreeData[i].cellType == 1) m_arrTreeData[i].visible = false;
		updateData();
		updateTreeSel();
	}

	public function expandAll():Void
	{
		m_nOldSel = m_nSelCell = -1;
		for(var i:Number = 0; i < m_arrTreeData.length; i++)
			if(m_arrTreeData[i].cellType == 1) m_arrTreeData[i].visible = true;
		updateData();
		updateTreeSel();
	}

	public function getTreeStruct(arrData:Array):Object
	{
		var root:Object = null;
		if(arrData.length > 0)
		{
			var node:Object = root = new Object();
			var prevItem:Object = null;
			for(var i:Number = 0; i < arrData.length; i++)
			{
				var item:Object = arrData[i];
				while(node.item && node.item.depth >= item.depth)
					node = node.parent;
				var subNode:Object = {item:item, parent:node};
				if(prevItem) prevItem.next = subNode;
				prevItem = subNode;
				if(!node.nodes) node.nodes = new Array();
				node.nodes.push(subNode);
				node = subNode;
			}
		}
		return root;
	}

	/**
	 * Updates list data from tree data. Removes/adds items
	 * according to their visibility.
	 */
	public function updateData():Void
	{
		m_arrData.splice(0);
		var clrInd:Number = -1;
		var depth, cellType:Number;
		for(var i:Number = 0; i < m_arrTreeData.length; i++)
		{
			var curItem:Object = m_arrTreeData[i];
			if(curItem.cellType != cellType || curItem.depth != depth)
			{ // increase color number and update color switchers
				depth = curItem.depth;
				cellType = curItem.cellType;
				clrInd++;
			}
			if(curItem.cellType == 1 && !curItem.visible)
			{ // hidden branch
				m_arrData.push(curItem);
				var curDepth:Number = curItem.depth;
				while(++i < m_arrTreeData.length)
					if(m_arrTreeData[i].depth <= curDepth) break;
					else m_arrTreeData[i].ind = -1;
				i--;
			}
			else // visible item
				m_arrData.push(curItem);
			curItem.clrInd = clrInd;
		}
		m_bUpdated = false;
	}

	public function showBranch(id:Number):Void
	{ // xxx don't update all array but its part
		if(m_arrTreeData[id].cellType == 1)
		{
			m_arrTreeData[id].visible = true;
			updateData();
		}
	}

	public function hideBranch(id:Number):Void
	{ // xxx don't update all array but its part
		if(m_arrTreeData[id].cellType == 1)
		{
			m_arrTreeData[id].visible = false;
			updateData();
		}
	}

	public function getSelData():Object
	{
		return (m_nSelCell > -1) ? super.getSelData() : m_arrTreeData[m_nSelLeaf];
	}

	//
	// PRIVATE METHODS
	//

	private function getTreeId(listId:Number):Number
	{
		for(var i:Number = 0; i < m_arrTreeData.length; i++)
			if(m_arrTreeData[i].ind == listId) return i;
		return -1;
	}

	private function updateList():Void
	{
		reworkCells();
		for(var i:Number = 0; i < m_arrCells.length; i++)
		{
			var item:Object = m_arrTreeData[getTreeId(m_arrData[m_nFirstVisible + i].ind)];
			m_arrCells[i].setData({
				cellType:item.cellType,
				depth:item.depth,
				visible:item.visible,
				clrInd:item.clrInd,
				first:item.first,
				last:item.last,
				hasChildren:item.hasChildren,
				lines:item.lines
			});
		}
		updateCells();
	}

	private function updateTreeSel():Void
	{
		//m_nOldSel = m_nSelCell = (m_nSelLeaf > -1) ? m_arrTreeData[m_nSelLeaf].ind : -1;
	}

	private function onChanged():Void
	{
		var id:Number = getTreeId(m_nSelCell);
		if(getSelData().cellType == 1)
		{
			m_arrTreeData[id].visible ? hideBranch(id) : showBranch(id);
			updateIndex();
			updateTreeSel();
			update();
			draw();
		}
		else m_nSelLeaf = id;
	}
}